#!/usr/bin/python3
"""Module to define fixture for unit tests."""
from charmhelpers.core import unitdata

import mock

import pytest


# If layer options are used, add this to prometheusgrokexporter
# and import layer in lib_prometheus_grok_exporter
@pytest.fixture
def mock_layers(monkeypatch):
    """Mock import layers."""
    import sys

    sys.modules["charms.promreg"] = mock.MagicMock()
    sys.modules["charms.layer"] = mock.Mock()
    sys.modules["reactive"] = mock.Mock()


@pytest.fixture
def mock_hookenv_config(monkeypatch):
    """Mock charm config."""
    import yaml

    def mock_config():
        cfg = {}
        yml = yaml.safe_load(open("./config.yaml"))

        # Load all defaults
        for key, value in yml["options"].items():
            cfg[key] = value["default"]

        # Manually add cfg from other layers
        # cfg['my-other-layer'] = 'mock'
        return cfg

    monkeypatch.setattr("lib_prometheus_grok_exporter.hookenv.config", mock_config)


@pytest.fixture
def mock_remote_unit(monkeypatch):
    """Mock remote unit."""
    monkeypatch.setattr(
        "lib_prometheus_grok_exporter.hookenv.remote_unit", lambda: "unit-mock/0"
    )


@pytest.fixture
def mock_charm_dir(monkeypatch):
    """Mock charm directory."""
    monkeypatch.setattr(
        "lib_prometheus_grok_exporter.hookenv.charm_dir", lambda: "/mock/charm/dir"
    )


@pytest.fixture
def mock_unit_db(monkeypatch):
    """Mock the key value store."""
    mock_kv = mock.Mock()
    mock_kv.return_value = unitdata.Storage(path=":memory:")
    monkeypatch.setattr("lib_prometheus_grok_exporter.unitdata.kv", mock_kv)


@pytest.fixture
def prometheusgrokexporter(
    mock_layers,
    tmpdir,
    mock_hookenv_config,
    mock_charm_dir,
    mock_unit_db,
    monkeypatch,
):
    """Mock the PrometheusGrokExporterHelper."""
    from lib_prometheus_grok_exporter import PrometheusGrokExporterHelper

    helper = PrometheusGrokExporterHelper()

    # Example config file patching
    cfg_file = tmpdir.join("example.cfg")
    with open("./tests/unit/example.cfg", "r") as src_file:
        cfg_file.write(src_file.read())
    helper.example_config_file = cfg_file.strpath

    # Any other functions that load helper will get this version
    monkeypatch.setattr(
        "lib_prometheus_grok_exporter.PrometheusGrokExporterHelper", lambda: helper
    )

    return helper
